local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local AddonName, a = ...
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

local c = BittensSpellFlashLibrary2

local function setBlastColor(color)
    a.spells["Arcane Blast"].FlashColor = color
end

local function flashBlast(color)
    setBlastColor(color)
    a.Flash("Arcane Blast")
end

local function flashNeutralAround(percent, manaRatio, warningRatio)
    if warningRatio and manaRatio >= warningRatio then
        setBlastColor("red")
    else
        setBlastColor(nil)
    end
    if manaRatio > percent / 100 then
        a.FlashAll("Arcane Blast", "Conjure Mana Gem")
    else
        a.Flash("Arcane Missiles", "Arcane Blast")
    end
end

local function flashArcane()
    a.FlashAll("Mage Armor", "Arcane Brilliance", "Dalaran Brilliance")
    if not s.InCombat() then
        a.Flash("Conjure Mana Gem")
        return
    end
    
    --------------------------------------------------------- mana calculations
    local mana = s.Power("player")
    local casting = s.CastingOrChannelingName(nil, "player")
    if casting ~= nil then
        mana = mana - s.SpellCost(casting)
    end
    
    local stackedMana = 
        78 * 15 * s.BuffStack(c.GetID("Combat Mind LFR"), "player")
            + 88 * 15 * s.BuffStack(c.GetID("Combat Mind"), "player")
            + 99 * 15 * s.BuffStack(c.GetID("Combat Mind Heroic"), "player")
    if s.Buff(c.FIVE_PERCENT_BUFFS, "player") then
        stackedMana = stackedMana * 1.05
    end
    local maxMana = s.MaxPower("player") - stackedMana
    if maxMana < a.UnstackedMaxMana then
        a.UnstackedMaxMana = maxMana
    end
    maxMana = a.UnstackedMaxMana + stackedMana
    local manaRatio = mana / maxMana
    
    --------------------------------------------------------- phase transitions
    local abStack = s.DebuffStack(c.GetID("Arcane Blast"), "player")
    local stStack = c.GetBuffStack("Stolen Time")
    if casting == s.SpellName(c.GetID("Arcane Blast")) then
        if abStack < 4 then
            abStack = abStack + 1
        end
        if stStack > 0 and stStack < 10 then
            stStack = stStack + 1
        end
    end
    
    if a.Phase == "neutral"
        and c.GetCooldown("Evocation") < a.GetConfig("burn_length") then
        
        a.Phase = "burn transition"
    end
    
    if a.Phase == "burn transition" and c.HasBuff("Arcane Power") then
        a.Phase = "burn"
    end
    
    if a.Phase == "burn"
        and manaRatio < a.GetConfig("evocate_percent") / 100 then
        
        a.Phase = "neutral transition"
    end
    
    if casting == s.SpellName(c.GetID("Evocation")) then
        a.Phase = "neutral"
    end
    
    ------------------------------------------------------------------ flashing
    if c.HasBuff("Arcane Power") then
        a.Flash("Mirror Images")
    else
        a.FlashAll("Flame Orb", "Presence of Mind")
    end
    
    if a.Phase == "neutral" then
        flashNeutralAround(a.GetConfig("missiles_percent"), manaRatio, 1)
        if stStack == 10
            and c.GetCooldown("Evocation") - a.GetConfig("burn_length") 
                > 37.5 then
        
            a.Flash("Arcane Power")
        end
        
    elseif a.Phase == "burn transition" then
        flashBlast("yellow")
        flashNeutralAround(a.GetConfig("missiles_percent"), manaRatio, 1)
        if abStack == 4 
            and maxMana - mana >= 12404 -- room for mana gem
            and (stStack == 0 or stStack == 10) then
            
            a.FlashAll("Arcane Power", "Mana Gem")
        end
        
    elseif a.Phase == "burn" then
        flashBlast()
        
    elseif a.Phase == "neutral transition" then
        if not a.Flash("Evocation") then
            flashNeutralAround(
                a.GetConfig("evocate_percent")
                    - (100 - a.GetConfig("missiles_percent")),
                manaRatio)
        end
    end
end

local function flashFrost()
    a.FlashAll(
        "Molten Armor", 
        "Summon Water Elemental", 
        "Mirror Images", 
        "Arcane Brilliance", 
        "Dalaran Brilliance")
    if not s.InCombat() then
        return
    end
    
    -- non-GCD cooldowns
    
    -- rotation
    a.Flash(
        "Frostfire Orb",
        "Deep Freeze",
        "Freeze",
        "Instant Frozen Frostfire Bolt",
        "Frozen Ice Lance",
        "Frostbolt",
        "Evocation")
end


s.Spam[AddonName] = function()
    c.Init(AddonName, a)
    c.Run(
        {flashArcane, 1, "arcane_off"}, 
        {flashFrost, 3, "frost_off"})
end
